import json

from pydantic import BaseModel, root_validator, ValidationError

from ieg_validators.cloudport.errors import InvalidCRD
from ieg_validators.cloudport.models.crd import CRDBase
from ieg_validators.cloudport.models.account.config import AccountConfig


class S3(BaseModel):
    name: str
    region: str
    processedPath: str
    createIngestKeys: bool


class Spec(BaseModel):
    name: str
    domain: str
    storage: str
    amgId: str
    customerName: str
    hostName: str
    s3: S3
    config: AccountConfig

    @root_validator(pre=True)
    def serialize(cls, value):
        try:
            value["config"] = AccountConfig(**json.loads(value["config"]))
        except ValidationError as e:
            raise InvalidCRD(
                "Invalid json found at spec:config in account CRD", e
            ) from e
        except Exception as e:
            raise e
        return value


class CRD(CRDBase):
    spec: Spec
