import json

from pydantic import BaseModel, Field, root_validator, ValidationError

from ieg_validators.cloudport.utils import load_yaml
from ieg_validators.cloudport.errors import InvalidCRD
from ieg_validators.cloudport.models.crd import CRDBase
from ieg_validators.cloudport.models.blip.config import CRDConfig
from ieg_validators.cloudport.models.blip.automation import CRDAutomation


class Deployment(BaseModel):
    helmVersion: str


class Customer(BaseModel):
    name: str
    hostName: str
    rootDomain: str
    adminEmail: str
    supportEmail: str


class Tags(BaseModel):
    amagiId: str | None
    phase: str | None
    geo: str | None


class Spec(BaseModel):
    deployment: Deployment
    customer: Customer
    tags: Tags | None
    config: CRDConfig
    cp_automation: CRDAutomation = Field(alias="cp-automation")

    @root_validator(pre=True)
    def serialize(cls, value):
        try:
            value["config"] = CRDConfig(**json.loads(value["config"]))
        except ValidationError as e:
            raise InvalidCRD("Invalid json found at spec:config in blip CRD", e) from e
        try:
            value["cp-automation"] = CRDAutomation(
                **load_yaml(value["cp-automation"], "pyyaml")
            )
        except Exception as e:
            raise InvalidCRD(
                "Invalid yaml found at spec:cp-automation in blip CRD", e
            ) from e
        return value


class CRD(CRDBase):
    spec: Spec
