import os
import sys

from pathlib import Path
from pydantic import ValidationError

from ieg_validators.cloudport.errors import InvalidCRD
from ieg_validators.cloudport.utils import read_yaml, write_md
from ieg_validators.cloudport.models import BlipCRD, AccountCRD, ChannelCRD


def validate_crd(component: str, file_path: Path):
    try:
        data = read_yaml(Path(file_path), 'ruamel')
        if component == 'automation':
            BlipCRD(**data)
        elif component == 'account':
            AccountCRD(**data)
        elif component == 'channel':
            ChannelCRD(**data)
        else:
            raise NotImplementedError(f'Validator for {component} not yet implemented')
    except ValidationError as e:
        raise InvalidCRD('', e) from e
    except Exception as e:
        raise e


def main():
    errored = 0
    content = ''
    for file_path in sys.argv[2:]:
        try:
            validate_crd(sys.argv[1], Path(file_path))
        except FileNotFoundError as e:
            raise e
        except Exception as e:
            content += f"#### {file_path}:\n{e}\n\n"
            write_md(Path(os.path.expanduser('~/errors.md')), content)
            errored = 1
    if errored:
        print(content)
        sys.exit(1)
